# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
To normalize a filename and ensure valid characters are used.
(stolen from Django: https://github.com/django/django/blob/master/django/utils/text.py#L439).
"""

import unicodedata
import re


def normalize_filename(filename):
    """
    Normalizes a filename.

    :param filename: filename to normalize
    :return: normalized filename
    """

    filename = unicodedata.normalize('NFKD', filename).encode('ascii', 'ignore').decode('ascii')
    filename = re.sub('[^\w\s-]', '', filename).strip()
    return re.sub('[-\s]+', '-', filename)
